/**
 ** File:   libconf.h
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
/*
 * File:   libconf.h
 * Author: M. de Jonge
 *
 * Description:
 * Header for libconf.c
 *
 * Functions and data types for configuration files
 *
 */


#ifndef _libconf_h
#define _libconf_h

#include <stdio.h>

#ifdef False
#undef False
#endif

#ifndef Bool
#ifdef True
#undef True
#endif
#ifdef Bool
#undef Bool
#endif

typedef enum
{
	False,
	True
} Bool;
#endif

typedef enum
{
   noErr = 1,
   sysErr,
   badConfFile,
   notFound
} confErr;

typedef struct cf_item
{
   char* key;
   char* value;
   struct cf_item* next;
} item;

typedef struct cf_grp
{
   char* grpName;
   item* items;
   struct cf_grp* next;
} grp;

	
	

typedef struct 
{
   confErr  err;
   Bool     case_significant;
   char*    fileName;
   grp*     groups;
   grp*     currGroup;
   item*    currItem;
} conf_file;




/* Open, Close, Write */
conf_file* open_conf( char* file );
Bool       close_conf( conf_file* cf );
Bool       write_conf( conf_file* cf );




/* 
 * Goto:  group/item
 * Query: current group/item, group/conf empty
 */
grp*       current_group( conf_file* cf );
item*      current_item( conf_file* cf );

grp*       goto_group( conf_file* cf, char* gName );
item*      goto_item( conf_file* cf, char* iName );

grp*       goto_first_group( conf_file* cf );
grp*       goto_next_group( conf_file* cf );

item*      goto_first_item( conf_file* cf );
item*      goto_next_item( conf_file* cf );

Bool       group_empty( conf_file* cf );
Bool       conf_empty( conf_file* cf );




/*
 * Get: groupName, itemKey, itemValue
 *      int, string
 */
char*      group_name( grp* Group );
char*      item_key( item* i );
char*      item_value( item* i );

char*      get_str( conf_file* cf, char* group, char* Item, char* Default );
int        get_int( conf_file* cf, char* group, char* Item, int Default );




/*
 * Set: string, int
 */
Bool       set_str( conf_file* cf, char* group, char* Item, char* value );
Bool       set_int( conf_file* cf, char* group, char* Item, int value );




/*
 * Create/Delete Group/Item
 */
grp*       create_new_group( conf_file* cf, char* gName );
item*      create_new_item( conf_file* cf, char* iName );

Bool       del_group( conf_file* cf, char* gName );
Bool       del_item( conf_file* cf, char* gName, char* iName );




/*
 * misc
 */
void       case_significant( conf_file* cf );
void       case_insignificant( conf_file* cf );
confErr    get_error( conf_file* cf );


#endif


/* EOF libconf.h */
