/**
 ** File:   dialI.c
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/Text.h>
#include <Xm/Separator.h>
#include <Xm/PushB.h>
#include <Xm/MwmUtil.h>
#include <Xm/MessageB.h>

#include "libdialogs.h"

#include <stdio.h>

 
static dialDialogInfoRec dialogInfo;

static Widget dialDialog;
static Widget phone_field;

static void dialDialogPopupCallback( Widget w, XtPointer p1, XtPointer p2 );
static void textEntryFocus( Widget w, XtPointer p1, XtPointer p2 );
static void textEntryUnfocus( Widget w, XtPointer p1, XtPointer p2 );
static void dialDialogCancel( Widget w, XtPointer p1, XtPointer p2 );
static void dialDialogOk( Widget w, XtPointer p1, XtPointer p2 );

Widget dialDialogCreate( Widget parent )
{
	Widget dialog_contents,
			 form;
	XmString label, ok_b, title;
	Arg args[10];
	int n;
	
	label = XmStringCreateLocalized( "   Please enter phone number to dial:   ");
	title = XmStringCreateSimple( "Phone Number" );
	ok_b = XmStringCreateLocalized( " Dial " );
	
	n = 0;
	XtSetArg( args[n], XmNautoUnmanage, False ); n++;
	XtSetArg( args[n], XmNmessageString, label ); n++;
	XtSetArg( args[n], XmNokLabelString, ok_b ); n++;
	XtSetArg( args[n], XmNwidth, 	335 ); n++;
	XtSetArg( args[n], XmNheight, 200 ); n++;
	XtSetArg( args[n], XmNdialogTitle, title ); n++;

	dialDialog = XmCreateMessageDialog( parent, "numberDialog", args, n);
	XmStringFree( label );
	XmStringFree( ok_b );
	XmStringFree( title );
	
	form = XtVaCreateManagedWidget( "numberForm",
		xmFormWidgetClass, dialDialog,
		NULL );
		
	dialog_contents = XtVaCreateManagedWidget( "numberRowColumn",
		xmRowColumnWidgetClass, form, 
		XmNnumColumns,       2,
      XmNpacking,          XmPACK_COLUMN,  
      XmNleftAttachment,   XmATTACH_FORM,  
      XmNrightAttachment,  XmATTACH_FORM,  
		XmNtopAttachment,		XmATTACH_POSITION,
		XmNtopPosition,        25,
		NULL );

	label = XmStringCreateSimple( "Number:" );
	XtVaCreateManagedWidget( "numberLabel",
		xmLabelWidgetClass, dialog_contents,
		XmNlabelString, label,
		NULL );
	XmStringFree( label );

	phone_field = XtVaCreateManagedWidget( "numberField",
		xmTextFieldWidgetClass, dialog_contents,
		XmNcolumns,          2,
		NULL );

	XtVaSetValues( XtParent(dialDialog),
		XmNmwmDecorations, MWM_DECOR_BORDER|MWM_DECOR_TITLE |MWM_DECOR_MENU,
		XmNmwmFunctions, MWM_FUNC_MOVE|MWM_FUNC_CLOSE,
		NULL );

	XtAddCallback( XtParent(dialDialog), XmNpopupCallback,
			dialDialogPopupCallback, NULL );
	XtAddCallback( phone_field, XmNfocusCallback, textEntryFocus, NULL );
	XtAddCallback( phone_field, XmNlosingFocusCallback, textEntryUnfocus, NULL );
	XtAddCallback( dialDialog, XmNcancelCallback, dialDialogCancel, NULL );
	XtAddCallback( dialDialog, XmNunmapCallback, dialDialogCancel, NULL );
	XtAddCallback( dialDialog, XmNokCallback, dialDialogOk, NULL );

	XtSetSensitive( 
		XmMessageBoxGetChild( dialDialog, XmDIALOG_HELP_BUTTON ), False );
	return dialDialog;
}

void dialDialogPopup()
{
	if( !XtIsManaged( dialDialog ) )
		XtManageChild( dialDialog );
	XtPopup( XtParent( dialDialog ), XtGrabNone );
}

void dialDialogPopdown()
{
	XtPopdown( XtParent( dialDialog ) );
}

static void dialDialogCancel( Widget w, XtPointer p1, XtPointer p2 )
{
	XtPopdown( XtParent( dialDialog ) );
	dialogInfo.cancelCallback();
}

static void dialDialogOk( Widget w, XtPointer p1, XtPointer p2 )
{
	static char number[100];
	char* n;
	n = XmTextGetString( phone_field );
	if( strlen( n ) > 0 )
	{
		strcpy( number, n );
		XtFree( n );
		XtPopdown( XtParent( dialDialog ) );
		dialogInfo.dialCallback( number );
	}
}

static void textEntryFocus( Widget w, XtPointer p1, XtPointer p2 )
{
	int i;
	i = XmTextGetLastPosition( w );
	XmTextSetSelection( w, 0, i, 0 );
}

static void textEntryUnfocus( Widget w, XtPointer p1, XtPointer p2 )
{
	int i;
	i = XmTextGetLastPosition( w );
	XmTextClearSelection( w, 0 );
}


static void dialDialogPopupCallback( Widget w, XtPointer p1, XtPointer p2 )
{
	int i;
	XmProcessTraversal( phone_field, XmTRAVERSE_CURRENT );
	i = XmTextGetLastPosition( phone_field );
	XmTextSetCursorPosition( phone_field, i );
}

void dialDialogInit( dialDialogInfoRec* info )
{
	dialogInfo.cancelCallback = info->cancelCallback;
	dialogInfo.dialCallback = info->dialCallback;
	XmTextFieldSetString( phone_field, info->number );
}

/* EOF dialI.c */
