/**
 ** File:   errorI.c
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
#include <X11/Intrinsic.h>
#include <Xm/MessageB.h>
#include <Xm/MwmUtil.h>

#include "libdialogs.h"

static Widget errorDialog;
static int errorDialogAnswer;

static void errorDialogResponse( Widget w, XtPointer p1, XtPointer p2 );

void errorDialogCreate( Widget parent )
{
	XmString title;
	Arg args[5];
	int n;
	
	title = XmStringCreateSimple( "Error:" );
	n = 0;
	XtSetArg( args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL ); n++;
	XtSetArg( args[n], XmNdialogTitle, title ); n++;
	XtSetArg( args[n], XmNmessageAlignment, XmALIGNMENT_CENTER ); n++;
	XtSetArg( args[n], XmNwidth, 335 ); n++;
	errorDialog = XmCreateErrorDialog( parent, "errorDialog", args, n );
	
	XmStringFree( title );
	XtUnmanageChild( XmMessageBoxGetChild( errorDialog, XmDIALOG_HELP_BUTTON) );
	XtUnmanageChild( XmMessageBoxGetChild( errorDialog, XmDIALOG_CANCEL_BUTTON) );

	XtVaSetValues( XtParent( errorDialog ),
		XmNmwmDecorations, MWM_DECOR_BORDER|MWM_DECOR_TITLE |MWM_DECOR_MENU,
		XmNmwmFunctions, MWM_FUNC_MOVE|MWM_FUNC_CLOSE,
		NULL );
	XtAddCallback( XtParent( errorDialog ), XmNpopdownCallback,
		errorDialogResponse, NULL );
}

void errorDialogPopup( char* msg )
{
	XmString xs;
	errorDialogCreate( __toplevel );
	
	xs = XmStringCreateLtoR( msg, XmFONTLIST_DEFAULT_TAG );
	XtVaSetValues( errorDialog, XmNmessageString, xs, NULL );
	XmStringFree( xs );
	
	errorDialogAnswer = 0;
	XtManageChild( errorDialog );
	XtPopup( XtParent( errorDialog ), XtGrabNone );

	while( errorDialogAnswer == 0 || XtAppPending( AppContext ) )
		XtAppProcessEvent( AppContext, XtIMAll);

	XtDestroyWidget( errorDialog );
}

void errorDialogPopdown()
{
	XtPopdown( errorDialog );
}

static void errorDialogResponse( Widget w, XtPointer p1, XtPointer p2 )
{
	errorDialogAnswer = 1;
}
/* EOF errorI.c */
