/**
 ** File:   loginI.c
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/Text.h>
#include <Xm/Separator.h>
#include <Xm/PushB.h>
#include <Xm/MwmUtil.h>
#include <Xm/MessageB.h>

#include "libdialogs.h"

static Widget loginDialog;
static Widget user_field;
static Widget pw_field;

static char* passwd = NULL;

static loginDialogInfoRec dialogInfo;

static void loginDialogCancel( Widget w, XtPointer p1, XtPointer p2 );
static void loginDialogOk( Widget w, XtPointer p1, XtPointer p2 );
static void loginDialogPopupCallback( Widget w, XtPointer p1, XtPointer p2 );
static void textEntryFocus( Widget w, XtPointer p1, XtPointer p2 );
static void textEntryUnfocus( Widget w, XtPointer p1, XtPointer p2 );
static void checkPasswd( Widget w, XtPointer, XtPointer );

Widget loginDialogCreate( Widget parent )
{
	Widget dialog_contents;
	XmString label, title;
	Arg args[10];
	int n;
	
	label = XmStringCreateLocalized( "   Please enter username and password:   ");
	title = XmStringCreateSimple( "Login" );
	
	n = 0;
	XtSetArg( args[n], XmNautoUnmanage, False ); n++;
	XtSetArg( args[n], XmNmessageString, label ); n++;
	XtSetArg( args[n], XmNwidth, 335 ); n++;
	XtSetArg( args[n], XmNheight, 200 ); n++;
	XtSetArg( args[n], XmNdialogTitle, title ); n++;

	loginDialog = XmCreateQuestionDialog( parent, "loginDialog", args, n);
	XmStringFree( label );
	XmStringFree( title );
		
	dialog_contents = XtVaCreateManagedWidget( "loginRowColumn",
		xmRowColumnWidgetClass, loginDialog, 
      XmNnumColumns,       2,
      XmNpacking,          XmPACK_COLUMN,  
      XmNleftAttachment,   XmATTACH_FORM,  
      XmNrightAttachment,  XmATTACH_FORM,  
		NULL );

	label = XmStringCreateSimple( "User:" );
	XtVaCreateManagedWidget( "loginUserLabel:" , 
		xmLabelWidgetClass, dialog_contents,
		XmNlabelString, label,
		NULL );
	XmStringFree( label );
	
	label = XmStringCreateSimple( "Password:" );
	XtVaCreateManagedWidget( "loginPasswordLabel", 
		xmLabelWidgetClass, dialog_contents,
		XmNlabelString, label,
		NULL );
	XmStringFree( label );

	user_field = XtVaCreateManagedWidget( "loginUserValue",
		xmTextFieldWidgetClass, dialog_contents,
		XmNcolumns,          2,
		NULL );

	pw_field = XtVaCreateManagedWidget( "loginPasswordField",
		xmTextFieldWidgetClass, dialog_contents,
		XmNcolumns,          2,
		NULL );

	XtVaSetValues( XtParent( loginDialog ),
		XmNmwmDecorations, MWM_DECOR_BORDER|MWM_DECOR_TITLE |MWM_DECOR_MENU,
		XmNmwmFunctions, MWM_FUNC_MOVE|MWM_FUNC_CLOSE,
		NULL );
	XtAddCallback( XtParent( loginDialog ), XmNpopupCallback,
		loginDialogPopupCallback, NULL );
	XtAddCallback( loginDialog, XmNcancelCallback, loginDialogCancel, NULL );
	XtAddCallback( loginDialog, XmNunmapCallback, loginDialogCancel, NULL );
	XtAddCallback( loginDialog, XmNokCallback, loginDialogOk, NULL );
	XtAddCallback( user_field, XmNfocusCallback, textEntryFocus, NULL );
	XtAddCallback( user_field, XmNlosingFocusCallback, textEntryUnfocus,NULL );	
	XtAddCallback( pw_field, XmNfocusCallback, textEntryFocus, NULL );
	XtAddCallback( pw_field, XmNlosingFocusCallback, textEntryUnfocus,NULL );	
	XtAddCallback( pw_field, XmNmodifyVerifyCallback, checkPasswd, NULL );

	XtSetSensitive(
		XmMessageBoxGetChild( loginDialog, XmDIALOG_HELP_BUTTON ), False );
	return loginDialog;
}

void loginDialogInit( loginDialogInfoRec* info )
{
	dialogInfo.cancelCallback = info->cancelCallback;
	dialogInfo.okCallback = info->okCallback;
	
	XmTextFieldSetString( user_field, info->user );
	XmTextFieldSetString( pw_field, info->pw );

}

static void loginDialogPopupCallback( Widget w, XtPointer p1, XtPointer p2 )
{
	int i;
	i = XmTextGetLastPosition( user_field );
	XmTextSetCursorPosition( user_field, i );
	if( i == 0 )
		XmProcessTraversal( user_field, XmTRAVERSE_CURRENT );
	else
		XmProcessTraversal( pw_field, XmTRAVERSE_CURRENT );
	i = XmTextGetLastPosition( pw_field );
	XmTextSetCursorPosition( pw_field, i );
}		
		
static void checkPasswd( Widget w, XtPointer p1, XtPointer call_data )
{
	char* new;
	int len;
	XmTextVerifyCallbackStruct* cbs;
	
	cbs = (XmTextVerifyCallbackStruct*)call_data;

	if( passwd == NULL )
		len = 0;
	else
		len = strlen( passwd );
	len -= cbs->endPos - cbs->startPos;
	len += 2;
	

	new = XtMalloc( len );

	if( passwd != NULL )
		strncpy( new, passwd, cbs->startPos );
	new[cbs->startPos] = '\0';
	strncat( new, cbs->text->ptr, cbs->text->length );
	new[cbs->startPos + cbs->text->length] = '\0';
	if( passwd != NULL )
		strcat( new, passwd + cbs->endPos );
	if( passwd != NULL )
		XtFree( passwd );
	passwd = new;

	for( len = 0; len < cbs->text->length; len++ )
		cbs->text->ptr[len] = '*';
}

void loginDialogPopup()
{
	if( !XtIsManaged( loginDialog ) )
		XtManageChild( loginDialog );
	XtPopup( XtParent( loginDialog ), XtGrabNone );
}

void loginDialogPopdown()
{
	XtPopdown( XtParent( loginDialog ) );
}

static void textEntryFocus( Widget w, XtPointer p1, XtPointer p2 )
{
   int i;
   i = XmTextGetLastPosition( w );
   XmTextSetSelection( w, 0, i, 0 );
}

static void textEntryUnfocus( Widget w, XtPointer p1, XtPointer p2 )
{
   int i;
   i = XmTextGetLastPosition( w );
   XmTextClearSelection( w, 0 );
}


static void loginDialogCancel( Widget w, XtPointer p1, XtPointer p2 )
{
   XtPopdown( XtParent( loginDialog ) );
   dialogInfo.cancelCallback();
}

static void loginDialogOk( Widget w, XtPointer p1, XtPointer p2 )
{
   static char  user[20];
	static char pw[20];
   char* u;
   int u_l, p_l;
   
   u = XmTextGetString( user_field );
	u_l = strlen( u );
	p_l = strlen( passwd );
	if( passwd == NULL )
		p_l = 0;

	if( u_l == 0 )
		return;
	
	if( p_l == 0 )
	{
		if( u_l > 0 )
			XtFree( u );
		return;
	}

	strncpy( user, u, sizeof(user));
	strncpy( pw,   passwd, sizeof(user)  );
	XtFree( u );
	XtPopdown( XtParent( loginDialog ) );
	dialogInfo.okCallback( user, pw );  
}   

/* EOF loginI.c */
