/**
 ** File:   loginMsgI.c
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/Separator.h>
#include <Xm/PushB.h>
#include <Xm/MwmUtil.h>
#include <Xm/MessageB.h>
#include <Xm/Frame.h>
#include <time.h>
#include <stdio.h>
#include <setjmp.h>

#include "libdialogs.h"

static loginMsgDialogInfoRec dialogInfo;

static Widget loginMsgDialog;
static Widget user_field;
static Widget time_field;

static XtIntervalId timeout = -1;

static void loginMsgDialogCancel( Widget w, XtPointer p1, XtPointer p2 );
static void loginMsgDialogPopupCallback( Widget w, XtPointer p1, XtPointer p2 );
static void loginMsgDialogPopdownCallback( Widget w, XtPointer p1, XtPointer p2 );
static void updateTime( XtPointer data, XtIntervalId* id );

Widget loginMsgDialogCreate( Widget parent )
{
	Widget dialog_contents,
			 form,
			 frame;
	XmString title, label;
	Arg args[10];
	int n;
	
	label = XmStringCreateLocalized( "   Loging into the network:   ");
	title = XmStringCreateSimple( "Login...");
	
	n = 0;
	XtSetArg( args[n], XmNwidth, 335 ); n++;
	XtSetArg( args[n], XmNheight, 200 ); n++;
	XtSetArg( args[n], XmNmessageString, label ); n++;
	XtSetArg( args[n], XmNdialogTitle, title ); n++;
	
	XtSetArg( args[n], XmNautoUnmanage, False); n++;
	loginMsgDialog = 
		XmCreateInformationDialog( parent, "loginMsgDialog", args, n);
	XmStringFree( label );
	XmStringFree( title );
	

	frame = XtVaCreateManagedWidget( "loginMsgFrame",
		xmFrameWidgetClass, loginMsgDialog,
		NULL );

	form = XtVaCreateManagedWidget( "loginMsgForm", 
		xmFormWidgetClass, frame, 
      XmNbottomOffset,		20,
      XmNtopOffset,			20,
		NULL );
		
	dialog_contents = XtVaCreateManagedWidget( "loginMsgRowColumn",
		xmRowColumnWidgetClass, form, 
      XmNnumColumns,       2,
      XmNpacking,          XmPACK_COLUMN,  
		XmNorientation,		XmVERTICAL,
      XmNleftAttachment,   XmATTACH_POSITION,  
      XmNrightAttachment,  XmATTACH_POSITION,  
		XmNtopAttachment,		XmATTACH_POSITION,
		XmNleftPosition,		30,
      XmNrightPosition,		70,
      XmNtopPosition,		20,
		XmNisAligned,			False,
      NULL );

	label = XmStringCreateLocalized( "User:" );	
	XtVaCreateManagedWidget( "loginMsgUserLabel" , 
		xmLabelWidgetClass, dialog_contents,
		XmNlabelString, label,
		XmNalignment, XmALIGNMENT_BEGINNING,
		NULL );
	XmStringFree(label);

	label = XmStringCreateLocalized( "Time:" );	
	XtVaCreateManagedWidget( "loginMsgTimeLabel" , 
		xmLabelWidgetClass, dialog_contents,
		XmNlabelString, label,
		XmNalignment, XmALIGNMENT_BEGINNING,
		NULL );
	XmStringFree(label);
	
	user_field = XtVaCreateManagedWidget( "loginMsgUserValue",
		xmLabelWidgetClass, dialog_contents,
		XmNalignment, XmALIGNMENT_END,
		NULL );

	time_field = XtVaCreateManagedWidget( "loginMsgTimeValue",
		xmLabelWidgetClass, dialog_contents,
		XmNalignment, XmALIGNMENT_END,
		NULL );

	XtVaSetValues( XtParent(loginMsgDialog),
		XmNmwmDecorations, MWM_DECOR_BORDER|MWM_DECOR_TITLE |MWM_DECOR_MENU,
		XmNmwmFunctions, MWM_FUNC_MOVE|MWM_FUNC_CLOSE,
		NULL );

	XtUnmanageChild( XmMessageBoxGetChild( loginMsgDialog, XmDIALOG_OK_BUTTON ) );
	XtUnmanageChild( XmMessageBoxGetChild( loginMsgDialog, XmDIALOG_HELP_BUTTON ) );
	
	XtAddCallback( XtParent(loginMsgDialog), XmNpopupCallback, 
			loginMsgDialogPopupCallback, NULL );
	XtAddCallback( XtParent(loginMsgDialog), XmNpopdownCallback,
			loginMsgDialogPopdownCallback, NULL );
	XtAddCallback( loginMsgDialog, XmNcancelCallback, loginMsgDialogCancel, NULL );
	XtAddCallback( loginMsgDialog, XmNunmapCallback, loginMsgDialogCancel, NULL );

	return loginMsgDialog;
}

void loginMsgDialogPopup()
{
	if( !XtIsManaged( loginMsgDialog ) )
		XtManageChild( loginMsgDialog );
	XtPopup( XtParent( loginMsgDialog ), XtGrabNone );
}

void loginMsgDialogPopdown()
{
	XtPopdown( XtParent( loginMsgDialog ) );

}

void loginMsgDialogInit( loginMsgDialogInfoRec* info )
{
	XmString xs;
	
	dialogInfo.cancelCallback = info->cancelCallback;
	dialogInfo.timeoutCallback = info->timeoutCallback;
	dialogInfo.timeout = info->timeout;
	xs = XmStringCreateLocalized( info->user );
	XtVaSetValues( user_field, XmNlabelString, xs, NULL );
	XmStringFree( xs );
	timeout = -1;
}

static void loginMsgDialogCancel( Widget w, XtPointer p1, XtPointer p2 )
{
	XtPopdown( XtParent( loginMsgDialog ) );
	dialogInfo.cancelCallback();
}

static void loginMsgDialogPopupCallback( Widget w, XtPointer p1, XtPointer p2 )
{
	updateTime( NULL, NULL );

}

static void loginMsgDialogPopdownCallback( Widget w, XtPointer p1, XtPointer p2 )
{
	if( timeout != -1 )
		XtRemoveTimeOut( timeout );
	timeout = -1;
}

static void updateTime( XtPointer data, XtIntervalId* id )
{
	char buf[20];
	struct tm* tm;
	XmString xs;
	if( id != NULL )
		dialogInfo.timeout--;
	if( dialogInfo.timeout == 0 )
	{
		XtPopdown( XtParent( loginMsgDialog ) );
		dialogInfo.timeoutCallback();
		return;
	}
			
	tm = gmtime( &dialogInfo.timeout );
	strftime( buf, sizeof(buf), "%M:%S", tm );
	
	xs = XmStringCreateLocalized( buf );
	XtVaSetValues( time_field, XmNlabelString, xs, NULL );
	XmStringFree( xs );

	timeout = XtAppAddTimeOut( AppContext, 1000, updateTime, NULL );
}

/* EOF loginMsgI.c */
