/**
 ** File:   liberror.c
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
#include <stdarg.h>
#include <stdio.h>
#include <time.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

#include "liberror.h"

#define errMsgBufLength 100

static void defaultErrorFunction( const char* fmt );
static char _errMsgBuf[errMsgBufLength];

static _errorFuncPtr errorFuncPtr = defaultErrorFunction;

static void defaultErrorFunction( const char* msg )
{
	fprintf( stderr, "%s\n", msg );
}


void error( const char* fmt, ... )
{
/* Does printf recognize the "%m" format string */
#ifndef PRINTF_HAS_M_FORMAT
	va_list ap;
	char* str;
	char* ptr2;
	int escaped;

	va_start( ap, fmt );
	
	escaped = 0;
	
	str = (char*)malloc( strlen( fmt ) + 1 );
	ptr2 = str;
	
	while( *fmt )
	{
		switch( *fmt )
		{
			case '%':
				if( *(fmt+1) == 'm' )
				{
					*ptr2 = '\0';
					str = (char* )realloc( str, strlen( str ) + 
										strlen( sys_errlist[errno] + 1 ) );
					strcat( str, sys_errlist[errno] );
					ptr2 = str + strlen( str );
					fmt++;
					fmt++;
				}
				else
				if( *(fmt+1) == '%' )
				{
					*ptr2++ = *fmt++;
					*ptr2++ = *fmt++;
					break;
				}
				else
					*ptr2++ = *fmt++;
				break;
			default:
				*ptr2++ = *fmt++;
				break;
		}
	}
	*ptr2 = '\0';
	
	vsnprintf( _errMsgBuf, sizeof( _errMsgBuf), str, ap );
	
	free( str );
	errorFuncPtr( _errMsgBuf );
	
	va_end( ap );

#else
	va_list ap;
	va_start( ap, fmt );
	vsnprintf( _errMsgBuf, sizeof( _errMsgBuf), fmt, ap );
	
	errorFuncPtr( _errMsgBuf );
	
	va_end( ap );
#endif
}

void setErrorFunc( _errorFuncPtr f )
{
	errorFuncPtr = f;
}

const _errorFuncPtr getErrorFunc()
{
	return errorFuncPtr;
}


const char* timeStamp()
{
   struct tm *tm;
   time_t now;
   static char time_stamp[60];
   char*  format = "[%c]";

   time( &now );
   tm = localtime( &now );
   
   strftime( time_stamp, sizeof( time_stamp ), format, tm );
   return time_stamp;
}

/* EOF liberror.c */
