/**
 ** File:   expect.c
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>

#include <liberror/liberror.h>
#include "expect.h"
#include "libmodem.h"

static char buf[512];
static int ind = 0;
static int bytes = 0;


static void getbyte( int fd )
{
	if( bytes == ind )
	{
		while( 1 )
		{
			if( modemRead( fd, &buf[bytes], 1 ) == -1 )
			{
				error( "read: %m" );
				exit( 1 );
			}
			if( buf[bytes] == '\0' || 
				 buf[bytes] == '\n' || 
				 buf[bytes] == '\r' 
				 )
				continue;
			break;
		}

		bytes++;
		buf[bytes] = '\0';
	}
	ind++;
}

/*
 * Shift over first character in buf and 
 * start over again
 */
static void shift()
{
	if( bytes > 0 )
		memmove( buf, buf + 1, --bytes );
	ind = 0;
}

void reset()
{
	ind = bytes = 0;
	buf[0] = '\0';
}

int expect( int fd, const char* str[], int matchMode )
{
	int i;
	reset();
	while( 1 )
	{
		getbyte( fd );
		i =0;
		while( str[i] != NULL )
		{
			if( toupper(str[i][ind-1]) == toupper(buf[ind-1] ) )
			{
				int stored_ind = ind;
				getbyte( fd );

				while( toupper(str[i][ind-1]) == toupper(buf[ind-1] ) )
				{
					if( str[i][ind] == '\0' )
						break;
					getbyte( fd );
				}
				if( str[i][ind] == '\0' )
				{
					reset();
					return i; /* string match */
				}
				/* string not match */
				ind = stored_ind;
			}
			if( matchMode == MATCH_BEGINNING )
				if( !isspace( buf[ind-1] ) )
					return -1;
			i++;
		}
		shift();
	}
}					

/* EOF expect.c */
