/**
 ** File:   modem.c
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
/*
 * dip		A program for handling dialup IP connecions.
 *		Modem driving module.  On systems that support the
 *		dial(3) package, we (should) use that.  Otherwise,
 *		we use a very rudimentary HAYES-type dialer.
 *
 * Version:	@(#)modem.c	3.3.3	08/16/93
 *
 * Author:      Fred N. van Kempen, <waltje@uWalt.NL.Mugnet.ORG>
 *		Copyright 1988-1993 MicroWalt Corporation
 *
 *		This program is free software; you can redistribute it
 *		and/or  modify it under  the terms of  the GNU General
 *		Public  License as  published  by  the  Free  Software
 *		Foundation;  either  version 2 of the License, or  (at
 *		your option) any later version.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>

#include <liberror/liberror.h>
#include "libmodem.h"

static modem_commands* commands = NULL;
static modemReadFuncPtr modemReadFunc = read;
static modemWriteFuncPtr modemWriteFunc = write;

int modemRead( int fd, void* data, size_t length )
{
	return modemReadFunc( fd, data, length );
}

int modemWrite( int fd, void* data, size_t length )
{
	return modemWriteFunc( fd, data, length );
}

void setModemReadFunc( modemReadFuncPtr newReadFunc )
{
	modemReadFunc = newReadFunc;
}

void setModemWriteFunc( modemWriteFuncPtr newWriteFunc )
{
	modemWriteFunc= newWriteFunc;
}

#define AT_EOL	"\r"

static int	mdm_setup = 0;			/* can we do anything?	*/

void mdm_set_commands( modem_commands* c )
{
	commands = c;
	mdm_setup = 1;
}


/* Dial a phone number. */
int
mdm_dial(int fd, char *number)
{
  char buff[128];

  if (mdm_setup == 1) {
	/* Setup to dial out. */
	sprintf(buff, "%s%s", commands->AT_INIT, AT_EOL);
	if( modemWrite( fd, buff, strlen(buff) ) == -1 )
	{
		error( "write: %m" );
		exit( 1 );
	}

	(void) sleep(1);
	/* Dial the phone number. */
	sprintf(buff, "%s%s%s", commands->AT_DIAL, number, AT_EOL);
	if( modemWrite( fd, buff, strlen(buff)) == -1 )
	{
		error( "write: %m" );
		exit( 1 );
	}
	return(0);
  }
  return(-1);
}

int mdm_hangup( int fd )
{
	char buf[128];
	if( mdm_setup == 1 )
	{
		if( modemWrite( fd, commands->AT_ATTN, 
				strlen( commands->AT_ATTN ) + 1) == -1 )
		{
			error( "write: %m" );
			exit( 1 );
		}
		
		(void) sleep(1);
		sprintf( buf, "%s%s", commands->AT_HANGUP, AT_EOL );
		if( modemWrite( fd, buf, strlen( buf ) ) == -1 )
		{
			error( "write: %m" );
			exit( 1 );
		}
		if( modemRead( fd, buf, sizeof(buf ) ) == -1 )
		{
			error( "read: %m" );
			exit( 1 );
		}
		
		return 0;
	}
	return -1;
}

int
mdm_reset(int fd )
{
  char buff[128];

  if (mdm_setup == 1) {
	/* Make the modem listen to us. */
	(void) sleep(1);
	if( modemWrite( fd, commands->AT_ATTN, 
				strlen( commands->AT_ATTN ) ) == -1 )
	{
		error( "write: %m" );
		exit( 1 );
	}
	(void) sleep(1);

	/* Reset the modem. */
	sprintf(buff, "%s%s", commands->AT_RSET, AT_EOL);
	if( modemWrite( fd, buff, strlen( buff ) ) == -1 )
	{
		error( "write: %m" );
		exit( 1 );
	}
	(void) sleep(3);
	return(0);
  }
  return(-1);
}

/* EOF modem.c */
