/**
 ** File:   libprotocol.c
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include <liberror/liberror.h>
#include "libprotocol.h"


typedef enum
{
	PROT_NORMAL_DATA,
	PROT_ERROR_DATA
} type_id;

typedef struct
{
	type_id	type;
	size_t length;
} protRec;


void protErrorDefaultFunc( const char* msg );

static protErrorFuncPtr protErrorFunc = protErrorDefaultFunc;

void protErrorDefaultFunc( const char* msg )
{
	error( msg );
}

protErrorFuncPtr getProtErrorFunc()
{
	return protErrorFunc;
}

void setProtErrorFunc( protErrorFuncPtr newErrorFunc )
{
	protErrorFunc = newErrorFunc;
}

int protSend( int fd, const void* data, size_t length )
{
	int result;
	protRec rec;

	rec.type = PROT_NORMAL_DATA;
	rec.length = length;
	
	result = write( fd, &rec, sizeof( rec ) );
	if( result == -1 )
		return result;

	result = write( fd, data, length );
	return result;
}

int protSendError( int fd, const void* data, size_t length )
{	
	int result;
	protRec rec;
	
	rec.type = PROT_ERROR_DATA;
	rec.length = length;
	
	result = write( fd, &rec, sizeof( rec ) );
	if( result < 0 )
		return -1;
	
	result = write( fd, data, length );
	return result;
}

int protReceive( int fd, void* data, size_t length )
{
	int result = -1;
	static protRec rec;
	static int got_all = 1;
	char* error_ptr;
	

	if( got_all )
	{
		result = read( fd, &rec, sizeof( rec ) );
		if( result < 0 )
			return -1;
	}
	
	switch( rec.type )
	{
		case PROT_NORMAL_DATA:
			result = read( fd, data, rec.length > length ? length : rec.length );
			if( result < 0 )
				return -1;
			if( result < rec.length )
				got_all = 0;
			else
				got_all = 1;
			rec.length -= result;
			break;
		case PROT_ERROR_DATA:
			error_ptr = (char*)malloc( rec.length );
			if( error_ptr  == NULL )
				return -1;
			result = read( fd, error_ptr, rec.length );
			if( result < 0 )
				return -1;
			protErrorFunc( error_ptr );
			free( error_ptr );
			got_all = 1;
			return -2;
	}

	return result;
}
/* EOF libprotocol.c */
