/**
 ** File:   libtcpip.h
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
#ifndef _libtcpip_h
#define _libtcpip_h

#include <stddef.h>
#include <netinet/in.h>

typedef enum
{
	UDP,
	TCP
} sockProtocol;

typedef struct
{
	int port;
	int fd;
	sockProtocol prot;
	struct sockaddr_in name;
} sockStruct;


int sockSetProtocol( sockStruct*, sockProtocol );
int sockSetPort( sockStruct*, int );
int sockSetPortToService( sockStruct*, char* service );
int sockServer( sockStruct* );
int sockClient( sockStruct*, const char* host );
int sockSetOption( sockStruct*, int option, int on );
int sockListen( sockStruct* );
int sockAccept( sockStruct*, sockStruct* );
int sockConnect( sockStruct* );
const char* sockConnectedTo( sockStruct* );
const char* sockPeerName( int );
int sockSend( sockStruct*, void* data, size_t size  );
int sockRecv( sockStruct*, sockStruct*, void* data, size_t size );
#endif

/* EOF libtcpip.h */
