/**
 ** File:   ttyutil.c
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
#include <string.h>
#include <sys/ioctl.h>

#include <libconf/libconf.h>
#include <modem_defs.h>
#include "ttyutil.h"
#include "ttyfuncs.h"

struct _tty_option
{
	char* name;
	int (*func)(char* );
};


static struct _tty_option tty_options[] = {
  { "databits",	tty_databits },
  { "parity",		tty_parity },
  { "speed",		tty_speed },
  { "stopbits",	tty_stopbits },
  { NULL, 			NULL }
};

static char* tty_name;

char* ttyDev()
{
	return tty_device_name();
}

int ttySetRaw( int fd )
{
   struct termios pty;
   if( ioctl( fd, TCGETS, &pty ) < 0 )
      return -1;

   tty_set_raw( &pty );

   if( ioctl( fd, TCSETS, &pty ) < 0 )
      return -1;
   
   return 0;
}

int ttyClose()
{
   return tty_close();
}


static char* ttyFindFreeTty( conf_file* cf )
{
	grp* group;
	group = goto_first_group( cf );
	while( group != NULL )
	{
		if( !tty_already_locked( group_name( group ) ) )
			return group_name( group );
		group = goto_next_group( cf );
	}
	return NULL;
}

static int ttyConfigure( char* dev, conf_file* cf )
{
	int status = 0;
	item* item;
	char* key;
	int i;
	
	goto_group( cf, dev );
	item = goto_first_item( cf );
	while( item != NULL )
	{
		key = item_key( item );
		i = 0;
		while( tty_options[i].name != NULL )
		{
			if( strcasecmp( tty_options[i].name, key ) == 0 )
				break;
			i++;
		}	
		if( tty_options[i].name != NULL )
		{
#ifdef DEBUG
	fprintf( stderr, "Calling: %s(%s)\n", tty_options[i].name,
		item_value(item) );
#endif
			status = tty_options[i].func( item_value( item ) );
			if( status < 0 )
			{
				error( "ttyConfigure: %m" );
				return -1;
			}
		}
		item = goto_next_item( cf );
	}
	return 0;
}

int ttyAvailable()
{
	conf_file* cf;
	int available;
	
	cf = open_conf( MODEM_CONF_FILE );
	if( cf == NULL )
		return 0;
	if( ttyFindFreeTty( cf ) == NULL )
		available = 0;
	else
		available = 1;
	close_conf( cf );
	
	return available;
}


int ttyOpen()
{
	conf_file* cf;
	int   ttyfd;
	
	cf = open_conf( MODEM_CONF_FILE );
	if( cf == NULL )
		return -1;
	tty_name = ttyFindFreeTty( cf );
	if( tty_name == NULL )
	{
		close_conf( cf );
		return -1;
	}
	tty_name = strdup( tty_name );
	
	ttyfd = tty_open( tty_name );
	if( ttyfd == -1 )
	{
		close_conf( cf );
		return -1;
	}
	
	if( ttyConfigure( tty_name, cf ) == -1 )
	{
		tty_close();
		close_conf( cf );
		return -1;
	}
	close_conf( cf );
	return ttyfd;
}	
	
char* ttyName()
{
	return tty_name;
}
/* EOF ttyutil.c */
