/**
 ** File:   log.c
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
#include <stdio.h>
#include <time.h>
#include <string.h>

#include "log.h"
#include <modem_defs.h>
#include <liberror/liberror.h>


FILE* ERR_LOG = stderr;
FILE* ACCESS_LOG = stderr;
FILE* ACCOUNT_LOG = stderr;

static char* progName;
static FILE	 old_err;

static int daemonMode = 0;

int startLog( char* _progName, int daemon )
{
	daemonMode = daemon;
	
	memmove( &old_err, stderr, sizeof(FILE) );
	if( daemonMode )
	{
		ERR_LOG = fopen( ERROR_LOG_FILE, "a" );
		if( ERR_LOG == NULL )
		{
			error( "open: \"%s\" %m", ERROR_LOG_FILE );
			exit( 1 );
		}
		ACCESS_LOG = fopen( ACCESS_LOG_FILE, "a" );
		if( ACCESS_LOG == NULL )
		{
			error( "open: \"%s\" %m", ACCESS_LOG_FILE );
			exit( 1 );
		}
		ACCOUNT_LOG = fopen( ACCOUNT_LOG_FILE, "a" );
		if( ACCOUNT_LOG == NULL )
		{
			error( "open: \"%s\" %m", ACCOUNT_LOG_FILE );
			exit( 1 );
		}
		*stderr = *ERR_LOG;
		memmove( stderr, ERR_LOG, sizeof(FILE) );

	}
	else
	{
		ERR_LOG = stderr;
		ACCESS_LOG = stderr;
	}
	progName = _progName;

	return 0;
}

int closeLog()
{
	memmove( stderr, &old_err, sizeof(FILE) );
	if( daemonMode )
	{
		fclose( ERR_LOG );
		fclose( ACCESS_LOG );
		fclose( ACCOUNT_LOG );
	}
	return 0;
}

/* EOF log.c */
