/**
 ** File:   main.c
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
#include <sys/socket.h>
#include <sys/wait.h>
#include <signal.h>
#include <errno.h>
#include <unistd.h>

#include <libtcpip/libtcpip.h>
#include <liberror/liberror.h>
#include "log.h"
#include "modemd.h"

static int DaemonMode = 1;
static int sock;
static char* progName;
static char hostName[40];

static void serverShutdown()
{
	shutdown( sock, 2 );
	closeLog();
	exit( 0 );
}

static void errMsg( const char* _msg )
{
	fprintf( ERR_LOG, "%s[%s] %s\n", timeStamp(), hostName, _msg );
}	
	
static void signal_handler( int sig )
{
	signal( sig, SIG_IGN );
	
	switch( sig )
	{
		case SIGTERM:
			error( "caught SIGTERM, shutting down" );
			serverShutdown();
			break;
	}
	signal( sig, signal_handler );
}

void main( int argc, char* argv[] )
{
	signal( SIGCHLD, SIG_IGN );
   signal( SIGHUP,  SIG_IGN );
   signal( SIGINT,  SIG_IGN );
   signal( SIGQUIT, SIG_IGN );
	signal( SIGTERM, signal_handler );
	
	progName = argv[0];
	gethostname( hostName, sizeof(hostName ) );	
	setErrorFunc( errMsg );

	if( argc > 1 )
		DaemonMode = 0;
	startup_daemon( DaemonMode );		

	startLog( argv[0], DaemonMode );
	server();
}
/* EOF main.c */
