/**
 ** File:   request.c
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
#include <stdlib.h>
#include "request.h"

#define MAX_REQUESTS 10
static request* requestList[MAX_REQUESTS];

void requestListInit()
{
	int i;
	for( i = 0; i < MAX_REQUESTS; i++ )
		requestList[i] = NULL;
}

void requestListReset()
{
	int i;
	for( i = 0; i < MAX_REQUESTS; i++ )
	{
		if( requestList[i] != NULL )
		{
			free( requestList[i] );
			requestList[i] = NULL;
		}
	}
}

int requestListAdd( request* r )
{
	int i;
	for( i = 0; i < MAX_REQUESTS; i++ )
		if( requestList[i] == NULL )
			break;
	
	if( i == MAX_REQUESTS )
		return -1;
	requestList[i] = r;
	return 0;
}

void requestListRemove( request_ID id )
{
	int i;
	i = requestListLookup( id );
	if( i == -1 )
		return;
	free( requestList[i] );
	requestList[i] = NULL;
}

int requestListLookup( request_ID id )
{
	int i;
	for( i = 0; i < MAX_REQUESTS; i++ )
		if( requestList[i] != NULL )
			if( requestList[i]->id == id )
				return i;
	return -1;
}

request* requestListGet( request_ID id )
{
	int i;
	i = requestListLookup( id );
	if( i == -1 )
		return NULL;
	return requestList[i];
}
/* EOF request.c */
