/**
 ** File:   xmlogin.c
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
#include <stdio.h>
#include <stdlib.h>
#include <setjmp.h>
#include <fcntl.h>
#include <unistd.h>
#include <getopt.h>

#include <libdialogs/libdialogs.h>
#include <libmodem/libmodem.h>
#include <libphonebook/libphonebook.h>
#include <liberror/liberror.h>
#include <modem_defs.h>
#include "login.h"

static void initStart( int argc, char* argv[] );
static void	initSetLogin( char* entry );

static void getLoginCancel();
static void getLoginDone( char*, char* );
static void doLoginInit( char*, char* );
static void doLoginError( char* );
static void doLoginTimeout();
static void doLoginCancel();
static void doLoginStart( char*, char* );

static int fd;
static jmp_buf env;
static int timeout;
static char* login = NULL;
static char* password = NULL;
static struct option long_options[] =
{
	{ "use",     1, 0, 'u' },
	{ 0,         0, 0,  0  }
};
         
static char* opt_string = "u:U:";

static void initStart( int argc, char* argv[] )
{
   int c;
   while( 1 )
   {
      int option_index = 0;
      c = getopt_long (argc, argv, opt_string, long_options, &option_index );
      if( c == -1 )
         break;
      switch( c )
      {
         case 'u':
         case 'U':
            initSetLogin( optarg );
            break;
         default:
            error( "Unrecognised option" );
            break;
      }
   }
}

static void initSetLogin( char* entry )
{
	phonebook_entry* e;
	phonebook* book;
	char* val;
	
	book = phonebookOpen( PHONEBOOK );
	if( book == NULL )
		return;
	e = phonebookGetEntryByName( book, entry );
	if( e != NULL )
	{
		val = loginValue( book, e );
		if( val != NULL )
			login = strdup( val );
		val = passwordValue( book, e );
		if( val != NULL )
			password = strdup( val );
	}
	phonebookClose( book );
}

		

static void getLoginStart( char* user, char* pw )
{
	loginDialogInfoRec info;

	info.user = user;
	info.pw = pw;
	info.cancelCallback = getLoginCancel;
	info.okCallback = getLoginDone;

	loginDialogInit( &info );
	loginDialogPopup();
}

static void getLoginCancel()
{
	exit( 1 );
}

static void getLoginDone( char* user, char* pw )
{
	doLoginInit( user, pw );
}

static void doLoginCancel()
{
	longjmp( env, 2 );
}

static void doLoginTimeout() 
{
	longjmp( env, 1 );
}


static void doLoginInit( char* user, char* pw )
{
	int status;
	loginMsgDialogInfoRec info;
	
	info.user = user;
	info.timeout = timeout;
	info.cancelCallback = doLoginCancel;
	info.timeoutCallback = doLoginTimeout;
	
	loginMsgDialogInit( &info );

	status = setjmp( env );
	switch( status )
	{
		case 0:
			loginMsgDialogPopup();
			doLoginStart( user, pw );
			return;
		case 1:
			doLoginError( "     Timeout       " );
			break;
		case 2:
			break;
	}
	loginMsgDialogPopdown();
	getLoginStart( user, pw );
}

static void doLoginStart( char* user, char* pw )
{	
	char buf[100];
	int ind;
	sprintf( buf, "%s\r", user );
	expect( fd, loginString, MATCH_ALL_TEXT );
	write( fd, buf, strlen( buf ) );
	sprintf( buf, "%s\r", pw );
	expect( fd, pwdString, MATCH_ALL_TEXT );
	write( fd, buf, strlen( buf ) );

	ind = expect( fd, failureString, MATCH_BEGINNING );
	if( ind != -1 )
	{
		loginMsgDialogPopdown();
		doLoginError( "    Login incorrect    " );
		getLoginStart( user, pw );
	}
	else
	{
		loginMsgDialogPopdown();
		exit( 0 );
	}
}

static void doLoginError( char* msg )
{
	errorDialogPopup( msg);
}
	
void main( int argc, char* argv[] )
{
	Widget toplevel;

	toplevel = interfaceCreate( &argc, argv, "XMlogin" );

	initStart( argc, argv );
	fd = 0;
	timeout = 60;
			
	loginDialogCreate( toplevel );
	loginMsgDialogCreate( toplevel );
	errorDialogCreate( toplevel );

	
	getLoginStart( login, password );
	
	interfaceRun();
}

	
/* EOF xmlogin.c */
