/**
 ** File:   connect.c
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>
#include <pwd.h>
#include <unistd.h>

#include <libtcpip/libtcpip.h>
#include <liberror/liberror.h>
#include <libprotocol/libprotocol.h>
#include <modem_defs.h>
#include "connect.h"
#include "../modemd/request.h" /* for request_ID */



static int modemd_broadcast( int fd )
{
	fd_set rfds;
	struct timeval timeout;
	int retval;
	
	while( 1 )
	{
		timeout.tv_sec = BROADCAST_TIMEOUT;
		timeout.tv_usec = 0;
	
		FD_ZERO( &rfds );
		FD_SET( fd, &rfds );
		retval = select( FD_SETSIZE, &rfds, NULL, NULL, &timeout );
	 	
	 	if( retval < 0 )
	 	{
	 		if( errno == EINTR )
	 			continue;
			error( "select: %m" );
		}
		return retval;
	}
}
									
int modemd_connect()
{
	uid_t me;
	struct passwd* pwd;
	sockStruct sock_tcp;
	sockStruct sock_udp;
	sockStruct server;
	const char* serverName;
	request_ID id;
	
	me = getuid();
	pwd = getpwuid( me );

	sockSetProtocol( &sock_tcp, TCP );
	sockSetProtocol( &sock_udp, UDP );
	
	if( sockSetPortToService( &sock_udp, UDP_SERVICE ) == -1 )
	{
		error( "Unknown service: %s, using default port: %d",
			UDP_SERVICE, UDP_DEFAULT_PORT );
		sockSetPort( &sock_udp, UDP_DEFAULT_PORT );
	}

	if( sockSetPortToService( &sock_tcp, TCP_SERVICE ) == -1 )
	{
		error( "Unknown service: %s, using default port: %d",
			TCP_SERVICE, TCP_DEFAULT_PORT );
		sockSetPort( &sock_tcp, TCP_DEFAULT_PORT );
	}

	if( sockClient( &sock_udp, BROADCAST_ADDR ) == -1 )
	{
		error( "udp %m" );
		exit( 1 );
	}
	
	if( sockSetOption( &sock_udp, SO_BROADCAST, 1 ) < 0 )
	{
		error( "sockSetOption %m" );
		exit( 1 );
	}
	if( sockSend( &sock_udp, pwd->pw_name, strlen( pwd->pw_name ) + 1) == -1 )
	{
		error( "sockSend %m" );
		exit( 1 );
	}
	while( 1 )
	{
		if( modemd_broadcast( sock_udp.fd ) == 0 )
		{
			error( "No servers found on the net" );
			exit( 1 );
		}
		if( sockRecv( &sock_udp, &server, &id, sizeof( id ) ) == -1 )
		{
			error( "recvfrom %m" );
			exit( 1 );
		}
		serverName = sockConnectedTo( &server );
		if( sockClient( &sock_tcp, serverName ) == -1 )
		{
			error( "tcp %m" );
			exit( 1 );
		}
		
		if( sockConnect( &sock_tcp ) == -1 )
		{
			error( "connect %m" );
			exit( 1 );
		}
		if( protSend( sock_tcp.fd, &id, sizeof(id) ) < 0 )
		{
			error( "write %m" );
			exit( 1 );
		}
		
		return sock_tcp.fd;
	}
}
			
/* EOF connect.c */
