/**
 ** File:   jobc.c
 ** Date:   Sun Oct 15 1995
 ** Author: Merijn de Jonge
 ** Email:  mdejonge@fwi.uva.nl
 **
 ** This file is part of the modem communication package.
 **
 ** Copyright (C) 1995 Merijn de Jonge.
 **
 ** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 ** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 ** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ** 
 ** Redistribution and use in source and binary forms are permitted provided
 ** that: (1) source distributions retain this entire copyright notice and
 ** comment, and (2) distributions may not be sold for profit on physical
 ** media such as disks, tapes, and CD-ROMS, without expressed written
 ** permission.
 ** 
 **/
#include <signal.h>
#include <sys/wait.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>

#include <liberror/liberror.h>
#include "jobc.h"

#define ARG_SIZE	20

static pid_t pid = -1;
int pgrp = -1;

static int buildCommand( int argc, char* argv[], char* command, char* pty )
{
	int i;
	char* ptr;
	ptr = command;
	i = 0;
	
	while( *ptr && i < argc - 3 )
	{
		while( (*ptr == ' ' ) || (*ptr == '\t' ) )
			*ptr++ = '\0';
		argv[i++] = ptr;
		
		while( (*ptr != '\0' ) && (*ptr != ' ') && (*ptr != '\t' ) )
			ptr++;
	}
	if( i == argc - 3 )
		return -1;
	argv[i++] = "-pty";
	argv[i++] = pty;
	argv[i] = NULL;
	return 0;
}
		
int jobControlNewJob( char* command, int ptySlaveDev )
{
	int i;
	char* argv[ARG_SIZE];
	
	if( buildCommand( ARG_SIZE, argv, command, ttyname( ptySlaveDev ) ) < 0 )
		return -1;
			
	pid = fork();
	switch( pid )
	{
		case 0:
			dup2( ptySlaveDev, 0 );
			dup2( ptySlaveDev, 1 );
			for( i = 3; i < getdtablesize(); i++ ) close( i );		
			execvp( argv[0], argv );
			exit( 0 );
		case -1:
			return -1;
		default:
			if( pgrp == -1 )
				pgrp = pid;
				if( setpgid( pid, pgrp ) < 0 )
				{
					error( "setpgid: %m" );
					exit(1);
				}
			return pid;
	}
}

void jobControlKillJobs()
{	
	if( pgrp != -1 && pid != -1 )
	{
		killpg( pgrp, SIGTERM );
	}

}

static void jobControlSignalHandler( int sig )
{
	pid_t dead_pid;
	int status;
	
	signal( sig, SIG_IGN );
	
	switch( sig )
	{
		case SIGCHLD:
			while( ( dead_pid = wait3( &status, WNOHANG | WUNTRACED, NULL ) ) > 0 )
			{
				if( dead_pid == pid )
					pid = -1;
			}
			exit( 0 );
	}
	signal( sig, jobControlSignalHandler );
}


void jobControlInit()
{
	signal( SIGCHLD, jobControlSignalHandler );
}

			
/* EOF jobc.c */
